# ESA (C) 2000-2022
# 
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

# XMMextractor_verify


import numpy as np
import os
import sys
import glob
from pysas.logger import TaskLogger as TL
from pysas.pyutils import pyutils



############################################
################ EPN files #################
############################################

def test_existance_pn_eventfiles(obs, exposure = ''):
    '''
    Checks if for a given expid exist event lists.
    
    Args:
        obs: the observation object.
        exposure: an exposure.
    
    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EPNlog = TL(os.path.join(obs.working_dir, 'EPNlog'))

    wd = os.getcwd()

    if not os.getcwd().endswith('EPN'):
        try:
            os.chdir(obs.EPN_dir)
        except FileNotFoundError:
            return (0, 'No EPN directory')

    if exposure == '':
        expid = ''
    else:
        if isinstance(exposure, str):
            expid = exposure[-4:]
        else:
            expid = exposure.expid[-4:]

    EPIC_evlist = glob.glob('*EPN*' + expid + '*ImagingEvts.ds')

    if len(EPIC_evlist) > 0:
        index = 0
        duration = []
        EPNlog.log('info', 'PN Image Event List found: {}'.format(EPIC_evlist))
        for i in EPIC_evlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duration.append(obj_duration)

        max_index = np.argmax(duration)
        pn_list = EPIC_evlist[max_index]
        EPNlog.log('info', 'Using event file with longest Observation Duration: {}'.format(pn_list))
        os.chdir(wd)
        return(True, obs.EPN_dir + '/' + pn_list)
    else:
        EPNlog.log('info', '#> PN Imaging Event List not found')
        os.chdir(wd)
        return(0, 'No file')


def test_existance_pn_timing_eventfiles(obs, exposure):
    '''
    Checks if for a given expid exist timing event lists.
    
    Args:
        obs: the observation object.
        exposure: an exposure.
    
    Output: 
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EPNlog = TL(os.path.join(obs.working_dir, 'EPNlog'))

    wd = os.getcwd()
    if not os.getcwd().endswith('GTI'):
        try:
            os.chdir(obs.GTI_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    if isinstance(exposure, str):
        expid = exposure[-4:]
    else:
        expid = exposure.expid[-4:]

    EPIC_evlist = glob.glob('pn' + expid + 'TimingEvts.ds')
    if len(EPIC_evlist) > 0:
        index = 0
        duration = []
        EPNlog.log('info', 'PN Timing Event List found: {}'.format(EPIC_evlist))
        
        for i in EPIC_evlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duraiton.append(obj_duration)

        max_index = np.argmax(duration)
        pn_list = EPIC_evlist[max_index]

        EPNlog.log('info', 'Using event file with longest Observation Duration: {}'.format(pn_list))
        os.chdir(wd)
        return(1, obsGTI_dir + '/' + pn_list)
    else:
        EPNlog.log('info', '#> PN Timing List not found')
        os.chdir(wd)
        return(0, 'No file')



#############################################
################# MOS files #################
#############################################

def test_existance_EMOS1(obs, exposure = ''):
    ''' 
    Checks if for a given expid exist event lists (EMOS1).
    
    Args:
        obs: an observation object.
        exposure: an exposure.
    
    Output: 
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EMOSlog = TL(os.path.join(obs.working_dir, 'EMOSlog'))

    wd = os.getcwd()

    if exposure == '':
        expid = ''
    else:
        if isinstance(exposure, str):
            expid = exposure[-4:]
        else:
            expid = exposure.expid[-4:]


    if not os.getcwd().endswith('EMOS'):
        try:
            os.chdir(obs.MOS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    EMOS1_evlist = glob.glob('*EMOS1*' + expid + '*ImagingEvts.ds')

    if len(EMOS1_evlist) > 0:
        index = 0
        duration = []
        EMOSlog.log('info', '#> EMOS1 Image Event List found: {}'.format(EMOS1_evlist))
        for i in EMOS1_evlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duration.append(obj_duration)

        max_index = np.argmax(duration)
        EMOS1_list = EMOS1_evlist[max_index]

        EMOSlog.log('info', '      Using event file with longest Observation Duration: {}'.format(EMOS1_list))
        os.chdir(wd)
        return(1, obs.MOS_dir + '/' + EMOS1_list)
    else:
        EMOSlog.log('info', '#> EMOS1 Imaging Event List not found')
        os.chdir(wd)
        return(0, 'No file')


def test_existance_EMOS1_timing(obs, exposure = ''):
    ''' 
    Checks if for a given expid exist timing event lists (EMOS1)
    
    Args:
        obs: an observation object.
        exposure: an exposure.
    
    Output: 
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EMOSlog = TL(os.path.join(obs.working_dir, 'EMOSlog'))

    wd = os.getcwd()

    if exposure == '':
        expid = ''
    else:
        if isinstance(exposure, str):
            expid = exposure[-4:]
        else:
            expid = exposure.expid[-4:]

    if not os.getcwd().endswith('GTI'):
        try:
            os.chdir(obs.GTI_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    EMOS1_timeevlist = glob.glob('*m1*' + expid + '*TimingEvts.ds')
    if len(EMOS1_timeevlist) > 0:
        index = 0
        duration = []
        EMOSlog.log('info', '#> EMOS1 timing Event List found: {}'.format(EMOS1_evlist))
        for i in EMOS1_timeevlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duraiton.append(obj_duration)

        max_index = np.argmax(duration)
        EMOS1_timelist = EMOS1_timeevlist[max_index]

        EMOSlog.log('info', '      Using event file with longest Observation Duration: {}'.format(EMOS1_timelist))
        os.chdir(wd)
        return(1, obs.GTI_dir + '/' + EMOS1_timelist)
    else:
        EMOSlog.log('info', '#> EMOS1 Timing Event List not found')
        os.chdir(wd)
        return(0, 'No file')


def test_existance_EMOS2(obs, exposure = ''):
    ''' 
    Checks if for a given expid exist event lists (EMOS2).
    
    Args:
        obs: the observation object.
        exposure: an exposure.

    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EMOSlog = TL(os.path.join(obs.working_dir, 'EMOSlog'))

    wd = os.getcwd()

    if exposure == '':
        expid = ''
    else:
        if isinstance(exposure, str):
            expid = exposure[-4:]
        else:
            expid = exposure.expid[-4:]


    if not os.getcwd().endswith('EMOS'):
        try:
            os.chdir(obs.MOS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    EMOS2_evlist = glob.glob('*EMOS2*' + expid + '*ImagingEvts.ds')
    if len(EMOS2_evlist) > 0:
        index = 0
        duration = []
        EMOSlog.log('info', '#> EMOS2 Image Event List found: {}'.format(EMOS2_evlist))
        for i in EMOS2_evlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duration.append(obj_duration)

        max_index = np.argmax(duration)
        EMOS2_list = EMOS2_evlist[max_index]

        EMOSlog.log('info', 'Using event file with longest Observation Duration: {}'.format(EMOS2_list))
        os.chdir(wd)
        return(1, obs.MOS_dir + '/' + EMOS2_list)
    else:
        EMOSlog.log('info', '#> EMOS2 Imaging Event List not found')
        os.chdir(wd)
        return(0, 'No file')


def test_existance_EMOS2_timing(obs, exposure = ''):
    ''' 
    Checks if for a given expid exist timing event lists (EMOS2)
    
    Args:
        obs: an observation object.
        exposure: an exposure.
    
    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    EMOSlog = TL(os.path.join(obs.working_dir, 'EMOSlog'))

    wd = os.getcwd()

    if exposure == '':
        expid = ''
    else:
        if isinstance(exposure, str):
            expid = exposure[-3:]
        else:
            expid = exposure.expid[-3:]

    if not os.getcwd().endswith('GTI'):
        try:
            os.chdir(obs.GTI_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    EMOS2_timeevlist = glob.glob('*m2*' + expid + '*TimingEvts.ds')
    if len(EMOS2_timeevlist) > 0:
        index = 0
        duration = []
        EMOSlog.log('info', '#> EMOS2 timing Event List found: {}'.format(EMOS2_evlist))
        for i in EMOS2_timeevlist:
            obj_duration = pyutils.get_key_word(i, 'DURATION', 1)
            duraiton.append(obj_duration)

        max_index = np.argmax(duration)
        EMOS2_timelist = EMOS2_timeevlist[max_index]

        EMOSlog.log('info', 'Using event file with longest Observation Duration: {}'.format(EMOS2_timelist))
        os.chdir(wd)
        return(1, obs.GTI_dir + '/' + EMOS2_timelist)
    else:
        EMOSlog.log('info', '#> EMOS2 Timing Event List not found')
        os.chdir(wd)
        return(0, 'No file')



#############################################
################# RGS files #################
#############################################

def test_existance_RGS1(obs):
    '''
    Checks if exist event lists (RGS1).

    Args:
        obs: the observation object.

    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    RGSlog = TL(os.path.join(obs.working_dir, 'RGSlog'))

    if not os.getcwd().endswith('RGS'):
        try:
            os.chdir(obs.RGS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    RGS1_evlist = glob.glob('*R1*EVENLI0000.FIT')
    if len(RGS1_evlist) > 0:
        RGSlog.log('debug', '   #> RGS1 Event List found')
        RGSlog.log('info', RGS1_evlist)
        if len(RGS1_evlist) > 1:
            RGSlog.log('info', '   #> Using only {}'.format(RGS1_evlist[0]))
        RGS1_event_list = RGS1_evlist[0]
        return (1, os.getcwd() + '/' + RGS1_event_list)
    else:
        RGSlog.log('info', '#> RGS1 Event List not found')
        return(0, 'No file')


def test_existance_RGS2(obs):
    '''
    Checks if exist event lists (RGS2).

    Args:
        obs: an Observation object.

    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    RGSlog = TL(os.path.join(obs.working_dir, 'RGSlog'))

    if not os.getcwd().endswith('RGS'):
        try:
            os.chdir(obs.RGS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    RGS2_evlist = glob.glob('*R2*EVENLI0000.FIT')
    if len(RGS2_evlist) > 0:
        RGSlog.log('info', '   #> RGS2 Event List found ')
        RGSlog.log('info', RGS2_evlist)
        if len(RGS2_evlist) > 1:
            RGSlog.log('info', '#> Using only {}'.format(RGS2_evlist[0]))
        RGS2_event_list = RGS2_evlist[0]
        return (1, os.getcwd() + '/' + RGS2_event_list)
    else:
        RGSlog.log('info', '#> RGS2 Event List not found')
        return(0, 'No file')


def test_existance_SRC_RGS1(obs):
    '''
    Checks if exist source files (RGS1).

    Args:
        obs: an observation object.
    
    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    RGSlog = TL(os.path.join(obs.working_dir, 'RGSlog'))

    if not os.getcwd().endswith('RGS'):
        try:
            os.chdir(obs.RGS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    RGS1_srclist = glob.glob('*R1*SRCLI_0000.FIT')
    if len(RGS1_srclist) > 0:
        RGSlog.log('info', '   #> RGS1 Source List found')
        RGSlog.log('info', RGS1_srclist)
        if len(RGS1_srclist) > 1:
            RGSlog.log('info', '   #> Using only {}'.format(RGS1_srclist[0]))
        RGS1_source_list = RGS1_srclist[0]
        return (1, os.getcwd() + '/' + RGS1_source_list)
    else:
        RGSlog.log('info', '#> RGS1 Source List not found')
        return(0, 'No file')


def test_existance_SRC_RGS2(obs):
    '''
    Checks if exist source files (RGS2).

    Args:
        obs: an observation object.

    Output:
        tuple. First element one (file) or zero (no file). Second element,
    a string with the file or 'No file'.
    '''

    RGSlog = TL(os.path.join(obs.working_dir, 'RGSlog'))

    if not os.getcwd().endswith('RGS'):
        try:
            os.chdir(XMMExtractor.RGS_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    RGS2_srclist = glob.glob('*R2*SRCLI_0000.FIT')
    if len(RGS2_srclist) > 0:
        RGSlog.log('info', '   #> RGS2 Source List found')
        RGSlog.log('info', RGS2_srclist)
        if len(RGS2_srclist) > 1:
            RGSlog.log('info', '   #> Using only {}'.format(RGS2_srclist[0]))
        RGS2_source_list = RGS2_srclist[0]
        return (1, os.getcwd() + '/' + RGS2_source_list)
    else:
        RGSlog.log('info', '#> RGS2 Source List not found')
        return(0, 'No file')


#############################################
################# OM  files #################
#############################################

def test_OM_event_files(obs):
    '''
    Checks for event list produced by omichain.

    Output:
        a tuple, with 0 or 1 (not found, found) and 'No file' or the name
    of the file.
    '''

    omlog = TL(os.path.join(obs.working_dir, 'OMlog'))

    if not os.getcwd().endswith('OM'):
        try:
            os.chdir(obs.OM_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    OM_event_list = glob.glob('P*OM*SIMAGE*.FIT')
    if len(OM_event_list) > 0:
        omlog.log('info', '#> OM Event List found [omichain]')
        omlog.log('info', OM_event_list)

        if len(OM_event_list) > 1:
            omlog.log('info', 'Using only {}'.format(OM_event_list[0]))
            OM_event = OM_event_list[0]
            return (1, os.getcwd() + '/' + OM_event)
    else:
        omlog.log('info', '#> OM Event List not found for omichain processing.')
        return (0, 'No file')


def test_OMF_event_files(obs):
    '''
    Checks for event list produced by omfchain.

    Output:
        a tuple, with 0 or 1 (not found, found) and 'No file' or the name
    of the file.
    '''

    omlog = TL(os.path.join(obs.working_dir, 'OMlog'))

    if not os.getcwd().endswith('OM'):
        try:
            os.chdir(obs.OM_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    OM_event_list = glob.glob('*OM*EVLIST*.FIT')
    if len(OM_event_list) > 0:
        omlog.log('info', '#> OM Event List found [omfchain]')
        omlog.log('info', OM_event_list)

        if len(OM_event_list) > 1:
            mlog.log('info', 'Using only {}'.format(OM_event_list[0]))
            OM_event = OM_event_list[0]
            return (1, os.getcwd() + '/' + OM_event)
    else:
        omlog.log('info', '#> OM Event List not found for omfchain processing.')
        return (0, 'No file')


def test_OMG_event_files(obs):
    '''
    Checks for event list produced by omgchain.

    Output:
        a tuple, with 0 or 1 (not found, found) and 'No file' or the name
    of the file.
    '''

    omlog = TL(os.path.join(obs.working_dir, 'OMlog'))

    if not os.getcwd().endswith('OM'):
        try:
            os.chdir(obs.OM_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    OM_event_list = glob.glob('p*OM*RIMAGE*.FIT')
    if len(OM_event_list) > 0:
        omlog.log('info', '#> OM Event List found [omgchain]')
        omlog.log('info', OM_event_list)

        if len(OM_event_list) > 1:
            xmmlog.info('    Using only {}'.format(OM_event_list[0]))
            OM_event = OM_event_list[0]
            return (1, os.getcwd() + '/' + OM_event)
    else:
        omlog.log('info','  #> OM Event List not found for omgchain processing.')
        return (0, 'No file')


def test_OMSRC_files(obs):
    '''
    Checks for Source files.

    Output:
        a tuple, with 0 or 1 (not found, found) and 'No file' or the name
    of the file.
    '''

    omlog = TL(os.path.join(obs.working_dir, 'OMlog'))

    if not os.getcwd().endswith('OM'):
        try:
            os.chdir(obs.OM_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    OM_event_list = glob.glob('*P*OM*SRLI*.FIT')
    if len(OM_event_list) > 0:
        omlog.log('info', '#> OM Source List found')
        omlog.log('info', OM_event_list)

        if len(OM_event_list) > 1:
            omlog.log('    Using only {}'.format(OM_event_list[0]))
            OM_event = OM_event_list[0]
        return (1, os.getcwd() + '/' + OM_event)
    else:
        omlog.log('info', '   #> OM Source List not found.')
        return (0, 'No file')


def test_OMSRCCombo_files(obs):
    '''
    Checks for Source combo files.

    Output:
        a tuple, with 0 or 1 (not found, found) and 'No file' or the name
    of the file.
    '''
    omlog = TL(os.path.join(obs.working_dir, 'OMlog'))

    if not os.getcwd().endswith('OM'):
        try:
            os.chdir(obs.OM_dir)
        except FileNotFoundError:
            return (0, 'No directory')

    OM_event_list = glob.glob('*P*OMCOMBOBSMLI*.FIT')
    if len(OM_event_list) > 0:
        omlog.log('info', '#> OM Source Combo List found')
        omlog.log('info', OM_event_list)

        if len(OM_event_list) > 1:
            omlog.log('info', 'Using only {}'.format(OM_event_list[0]))
            OM_event = OM_event_list[0]
            return (1, os.getcwd() + '/' + OM_event)
        else:
            OM_event = OM_event_list[0]
            omlog.log('info', 'Using {}'.format(OM_event))
            return (1, os.getcwd() + '/' + OM_event)
    else:
        omlog.log('info', '#> OM Source Combo List not found.')
        return (0, 'No file')



#############################################
############## Spectra   files ##############
#############################################


def check_spectra_files(instr, exposure):
    '''
    Check for the existence of EPIC event files.
    
    Args:
        instr: the instrument.
        exposure: the exposure.

    Output: 
        1 if EPIC file is found, 0 otherwise.
    '''

    if instr == 'EMOS1':
        test = test_existance_EMOS1(obs, exposure.expid[-4:])
    elif instr == 'EMOS2':
        test = test_existance_EMOS2(obs, exposure.expid[-4:])
    elif instr == 'EPN':
        test = test_existance_pneventfiles(obs, exposure.expid[-4:])
    else:
        test = (0, 'No file')

    return test


#############################################
################# EPN files #################
#############################################

def check_GTI_files(instr, exposure):
    '''
    Check for the existance of EPIC timing event files (non-filtered).
    
    Args:
        instr: the instrument.
        exposure: the exposure.

    Output:
        1 if found, 0 otherwise.
    '''

    if instr == 'EMOS1':
        test = test_existance_EMOS1_timing(obs, exposure.expid[-4:])
    elif instr == 'EMOS2':
        test = test_existance_EMOS2_timing(obs, exposure.expid[-4:])
    elif instr == 'EPN':
        test = test_existance_pn_timing_eventfiles(obs, exposure.expid[-4:])
    else:
        test = (0, 'No file')

    return test
